/*

interrupt routine

*/

//task counters used to tell when a task is due
unsigned int task0_counter=0;
unsigned int task1_counter=0;
unsigned int task2_counter=0;
unsigned int task3_counter=0;
unsigned int task4_counter=0;
unsigned int task5_counter=0;
unsigned int task6_counter=0;

//this tells when a task is going to happen again

//for task 2 (frequency of update)
unsigned int task2_counter_max=500;	//defaults to 4Hz

//for task 5 (pwm to control brightness of leds)
unsigned int task5_counter_max=0;
unsigned char flip_flop=0;
unsigned char brightness_duty_cycle;

//this enables/disables a task
volatile unsigned char task0_enable=TRUE;
volatile unsigned char task1_enable=TRUE;
volatile unsigned char task2_enable=TRUE;
volatile unsigned char task3_enable=TRUE;
volatile unsigned char task4_enable=TRUE;
volatile unsigned char task5_enable=TRUE;
volatile unsigned char task6_enable=TRUE;

//this allows tasks triggered by interrupt to run in the background in main
volatile unsigned char task0_go=FALSE;
volatile unsigned char task1_go=FALSE;
volatile unsigned char task2_go=FALSE;
volatile unsigned char task3_go=FALSE;
volatile unsigned char task4_go=FALSE;
volatile unsigned char task5_go=FALSE;
volatile unsigned char task6_go=FALSE;

void setup_interrupts(void)
{
	//set up tmr1	to interrupt every 500us

	TMR1CS=0;
	T1CKPS0=0;
	T1CKPS1=0;
	//T1OSCEN=1;

	//500 x4 = 2000 ticks, 0xFFFF - 2000 = 0xf82f, then 2000 ticks until rollover at 0xffff
	//to test, use simulator to find that its exactly correct

	#define	TICKS_BETWEEN_INTERRUPTS	2000
	#define	INTERRUPT_OVERHEAD	19

	#define TMR1RESET (0xFFFF-(TICKS_BETWEEN_INTERRUPTS-INTERRUPT_OVERHEAD))
	#define TMR1RESET_HIGH TMR1RESET >> 8
	#define TMR1RESET_LOW TMR1RESET & 0xFF

	TMR1ON=0;
	TMR1H=TMR1RESET_HIGH;
	TMR1L=TMR1RESET_LOW;
	TMR1ON=1;	

	TMR1IF=0;
	TMR1IE=1;
	PEIE=1;
	GIE=1;	
}

//pauses all interrupts
void pause_interrupts(void)
{
	task0_enable=FALSE;
	task1_enable=FALSE;
	task2_enable=FALSE;
	task3_enable=FALSE;
	task4_enable=FALSE;	
}

//restarts all interrupts
void go_interrupts(void)
{
	//reset task counters 
	task0_counter=0;
	task1_counter=0;
	task2_counter=0;
	task3_counter=0;
	task4_counter=0;

	task0_enable=TRUE;
	task1_enable=TRUE;
	task2_enable=TRUE;
	task3_enable=TRUE;
	task4_enable=TRUE;	
}

void interrupt isr(void)
{
	//one tick every 0.5us at 16Mhz
	if (TMR1IF)
	{
		TMR1IF=0;

		TMR1ON=0;
		TMR1H=TMR1RESET_HIGH;
		TMR1L=TMR1RESET_LOW;
		TMR1ON=1;

		task0_counter++;
		if (task0_counter>=1)
		{
			task0_counter=0;
			if (task0_enable==TRUE)
			{

				//check for serial in @9600 baud, at least every 833us
				//put this into the serial_in ring buffer
				if (getch_free())
				{
					serial_in_put(getch_interrupt());
				}
			}
		}

		task1_counter++;
		if (task1_counter>=2)
		{
			task1_counter=0;
			if (task1_enable==TRUE)
			{
				//generate serial out every 1ms
				//this copies contents of serial_out ring buffer to serial port
				if (!serial_out_empty)
				{
					putch_interrupt(serial_out_get());
				}
			}
		}

		task2_counter++;
		if (task2_counter>=task2_counter_max)	//every 500 ticks of 0.5ms = 250ms
		{
			task2_counter=0;
			if (task2_enable==TRUE)
			{
				//every 250ms take 3ms to update debt and display
				//do this in main so serial checking/generating is preserved
				task2_go=TRUE;
			}
		}
/*
		task3_counter++;
		if (task3_counter>=2000)
		{
			task3_counter=0;
			if (task3_enable==TRUE)
			{

				//every 1000ms fill up serial_out ring buffer with current debt digit[] and add[] info
				//do this in main so serial checking/generating is preserved
				task3_go=TRUE;
			}
		}
*/
		task4_counter++;
		if (task4_counter>=2200)
		{
			task4_counter=0;
			if (task4_enable==TRUE)
			{

				//every 1100ms check the input serial ring buffer for 'D' or 'A', meaning get
				//	update of current debt[] and add[] information
				//do this in main so serial checking/generating is preserved
				task4_go=TRUE;
			}
		}
/*		
		if (task5_counter<task5_counter_max)
		{
			task5_counter++;
		}
		else
		{
			task5_counter=0;
			if (task5_enable==TRUE)
			{

				//at 100Hz strobe the oe bar pin, at varying duty cycles,
				//	to get differing brightnesses

				//to control pwm duty cycle, vary brightness_duty_cycle
				//	duty cycle=brightness_duty_cycle/22
				//	brightness_duty_cycle varies 1 to 19, equivalent to 5% to 95%
				brightness_duty_cycle=19;

				flip_flop=1-flip_flop;	//this variable goes 1-0-1-0 etc
				if (flip_flop==1)
				{
					//turn on display
					oe_bar_lo;
					task5_counter_max=brightness_duty_cycle;
				}
				else
				{
					//turn off display	
					oe_bar_hi;
					task5_counter_max=20-brightness_duty_cycle;
				}
			}
		}
*/
		task6_counter++;
		if (task6_counter>=200)
		{
			task6_counter=0;
			if (task6_enable==TRUE)
			{

				//every 100ms check the power supply to see if there is power
				//	if there is none, tri-state all the inputs or else set up
				//	the ports to output as per normal
				//	this is so when power is off, pic isnt driving the 5891 buffers
				if (RB0==1)
				{
					//see convert.c
					setup_digit_shift_ports();
				}
				else
				{
					tristate_digit_shift_ports();
				}
			}
		}

	}	//if (TMR1IF)
}
